;;;; remote-mailbox-adapter.scm
;;;; Kon Lovett, Sep '17

(module remote-mailbox-adapter

(;export
  serializer deserializer
  serializer-output deserializer-input
  serialize deserialize)

(import scheme
  (chicken base)
  (chicken type)
  (prefix s11n s11n::)
  (only moremacros define-warning-parameter)
  (only type-errors-basic warning-argument-type)
  condition-utils)

(: serializer (#!optional (or false (* #!optional output-port -> void)) -> (* #!optional output-port -> void)))
(: deserializer (#!optional (or false (#!optional input-port -> *)) -> (#!optional input-port -> *)))
(: deserializer-input (#!optional (or false input-port) -> input-port))
(: serializer-output (#!optional (or false output-port) -> output-port))

(: serialize (* #!optional output-port -> void))
(: deserialize (#!optional input-port -> *))

;;;

;;

(define (serializer? x) (or (not x) (procedure? x)))
(define (deserializer? x) (or (not x) (procedure? x)))
(define (deserializer-input? x) (or (not x) (input-port? x)))
(define (serializer-output? x) (or (not x) (output-port? x)))

(define-warning-parameter serializer s11n::serialize serializer
                          ;ugh, automagic identifier injection
                          (unless obj (set! obj s11n::serialize)) )

(define-warning-parameter deserializer s11n::deserialize deserializer
                          ;ugh, automagic identifier injection
                          (unless obj (set! obj s11n::deserialize)) )

(define-warning-parameter deserializer-input (current-input-port) deserializer-input
                          ;ugh, automagic identifier injection
                          (unless obj (set! obj (current-input-port))) )

(define-warning-parameter serializer-output (current-output-port) serializer-output
                          ;ugh, automagic identifier injection
                          (unless obj (set! obj (current-output-port))) )

(define (serialize x #!optional (port (serializer-output)))
  ((serializer) x port) )

(define (deserialize #!optional (port (deserializer-input)))
  ((deserializer) port) )

) ;module remote-mailbox-adapter
